/*
 * sbug.c - Scriptable deBUGger
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dwarfx.h"

#include "util.h"
#include "stmt.h"
#include "cpp.h"
#include "sbug.h"


extern void open_elf(const char *name);
extern const void *mmap_file(const char *name,int *size);
extern int yyparse(void);
extern struct stmt *script;
extern const char *default_script;

struct dwx_obj *objects = NULL;
extern int debug;
int sbug = 0;

static int verbose = 0;


/* ----- Argument processing ----------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s script_file [-d] [-v] [cpp_option ...] "
      "sprogram [arg ...]\n",name);
    fprintf(stderr,"  -d  enable debugging output\n");
    fprintf(stderr,"  -v  verbose operation\n");
    fprintf(stderr,"  cpp_option  -Idir, -Dname[=value], or -Uname\n");
    exit(1);
}


int sbug_main(int argc,char **argv)
{
    static struct stmt exit_0;
    struct dwx_obj **link = &objects;
    int c;

    sbug = 1;
    while ((c = getopt(argc,argv,"dvD:U:I:")) != EOF)
	switch (c) {
	    static char opt[3] = "-?";

	    case 'd':
		debug = 1;
		break;
	    case 'v':
		verbose = 1;
		break;
	    case 'D':
	    case 'U':
	    case 'I':
		opt[1] = c;
		add_cpp_arg(stralloc(opt));
		add_cpp_arg(optarg);
		break;
	    default:
		usage(*argv);
	}
    if (argc < optind+2) usage(*argv);
    if (verbose)
	fprintf(stderr,"Loading symbols from file %s\n",argv[optind]);
    open_elf(argv[optind+1]);
    while (1) {
	const void *p;

	p = dwarf_comp_unit();
	if (!p) break;
	*link = dwx_read_objects(p);
	link = &(*link)->next;
    }
    init_types();
    if (!strcmp(argv[optind],"-")) {
	if (verbose) fprintf(stderr,"Executing script from standard input\n");
	run_cpp_on_file(NULL);
    }
    else {
	if (verbose)
	    fprintf(stderr,"Executing script from file %s\n",argv[optind]);
	run_cpp_on_file(argv[optind]);
    }
    (void) yyparse();
    reap_cpp();
    launch_subprocess(argv+optind+1);
    execute(script);
    exit_0.u.exit.expr = NULL;
    stmt_exit(&exit_0);
    return 0; /* not reached */
}
